% Author: Frank Haring, Stud.nr. 4416937, Date 14-01-2020
clc, clear, close all

%% Load data from selected folder
DataList = dir(strcat(uigetdir,'\*.mat')); % list with .mat files in the selected folder.
AllData = struct(); 

for i = 1:length(DataList) % Extract data from the .mat files into 1 structure
    Data = load(fullfile(DataList(i).folder,DataList(i).name)); 
    Fields = fieldnames(load(fullfile(DataList(i).folder,DataList(i).name)));
    AllData.(Fields{1})(i)=Data.(Fields{1});
end
AllData = AllData.(Fields{1});
[AllData.names] = DataList.name; % add filenames to the structure

clear i Fields Data 
%% Select field to be used.
UsedFields = {'names','time_s','sensor_V','position_mm'}; % field names required for processing
UsedData = struct();

 for i = 1:length(DataList)
     for j =1:length(UsedFields)
        UsedData(i).(UsedFields{j}) = AllData(i).(UsedFields{j});
     end
 end
 
clear i j DataList UsedFields
 %% Cleaning Names (remove auto generated timestamp form the file names)
% VariableList{1,length(UsedData)} = [];
 VariableList = strings;
   for i = 1:length(UsedData)
    N = strsplit(UsedData(i).names,'_'); % split name in parts;
    UsedData(i).names = strrep(string(strjoin(N(1:(end-1)))),' ','_'); %(removes auto generated time stamp from the name of the data)
    VariableList(i) = strrep(string(strjoin(N(1:(3)))),' ','_'); % creates variable namelist (ignores repetitions and sample numbers)
   end 

clear i N
%% Remove noise using a unweighted moving avarage filter
fwfv = [21,41]; % Filter window voltage (second value is for the PVA tests only)
fwfp = [21,41]; % Filter window position (second value is for the PVA tests only)

  for i = 1:length(UsedData)
      if contains(UsedData(i).names,'VA')  
        UsedData(i).sensor_V = maf(UsedData(i).sensor_V,fwfv(2))';  
        UsedData(i).position_mm = maf(UsedData(i).position_mm,fwfp(2))';
      else
        UsedData(i).sensor_V = maf(UsedData(i).sensor_V,fwfv(1))';
        UsedData(i).position_mm = maf(UsedData(i).position_mm,fwfp(1))';
      end    
  end 

clear i fwfv fwfp
 %% Convert the signal from Volt to Newtons:
Offv = 100; % Number of samples used to remove offset.
C1 = [3.7581 ,3.8060 ,3.8106, 3.8055]; % linear conversions constants determined for the 9N sensor [8.1654 ,3.7581,3.8060,3.8106]
C2 = [0,0,0,0]; %[0.1654, 0.0715, 0.0238, 0.0567]; % linear constants determined for the 9N sensos [0.1654, 0.0715, 0.0238, 0.0567]

  for i = 1:length(UsedData)
      % Dependend on the test different constants are to be used, determined by calibration.
      if contains(UsedData(i).names,'VA')  
          UsedData(i).force = C1(2).*UsedData(i).sensor_V - C2(2);
      elseif contains(UsedData(i).names,['DM10','DM20'])
          UsedData(i).force = C1(3).*UsedData(i).sensor_V - C2(3);
      elseif contains(UsedData(i).names,['DM30','GL'])
          UsedData(i).force = C1(4).*UsedData(i).sensor_V - C2(4);
      else
          UsedData(i).force = C1(1).*UsedData(i).sensor_V - C2(1);
      end
      
       UsedData(i).force = (UsedData(i).force - mean(UsedData(i).force(1,1:Offv))); % removes offset.
       UsedData(i).position_mm = (UsedData(i).position_mm - mean(UsedData(i).position_mm(1,1:Offv))); % removes offset.
      [UsedData(i).maxforce,UsedData(i).maxFindex] = max(UsedData(i).force(:)); % Extract maximum force. 
  end

clear i C1 C2 Offv
%% Statistical analysis 1:( effect internal structures, flat adhesives on different substrates) 
 Substrate = {'GL','DM10','DM20','DM30'}; % substrate variable {'GL','CL','VA','DM10','DM20','DM30'}
 IntStruc = {'I1', 'I2', 'I3', 'I4'}; % Internal structure variable {'I1', 'I2', 'I3', 'I4'}
 ExtStruc = {'P'}; % External structure variable {'P', 'H', 'F'}
 
 [g,MaxF] = AnovaGrouping(Substrate,IntStruc,ExtStruc,UsedData);
 g1 = g(1,:);
 g2 = g(2,:);
 g3 = g(3,:);
 
 % Anova comparing, remove any grouping(g1,g2,g3) that containes only one
 % variable, adjust multcompare dimention acordingly
[~,Tbl1,Stats,~]=anovan(MaxF,{g2,g1},'model','full','varnames',{'Internal structure','Substrate'},'Display','off'); 
[C1,M1,~,Gnames1] = multcompare(Stats,'dimension',[1,2],'Display','off');  

clear  MaxF g g1 g2 g3 Stats  Substrate IntStruc ExtStruc
%% Statistical analysis 2:(surface structures samples (I4) on different substrates) 
 Substrate = {'GL','DM10','DM20','DM30'}; % substrate variable {'GL','CL','VA','DM10','DM20','DM30'}
 IntStruc = {'I4'}; % Internal structure variable {'I1', 'I2', 'I3', 'I4'}
 ExtStruc = {'P', 'H', 'F'}; % External structure variable {'P', 'H', 'F'}
 
 [g,MaxF] = AnovaGrouping(Substrate,IntStruc,ExtStruc,UsedData);
 g1 = g(1,:);
 g2 = g(2,:);
 g3 = g(3,:);
 
 % Anova comparing, remove any grouping(g1,g2,g3) that containes only one
 % variable, adjust multcompare dimention acordingly
[~,Tbl2,Stats,~]=anovan(MaxF,{g1,g3},'model','full','varnames',{'Surface structure','Substrate'},'Display','off'); 
[C2,M2,~,Gnames2] = multcompare(Stats,'dimension',[1,2],'Display','off'); 

clear  MaxF g g1 g2 g3 Stats  Substrate IntStruc ExtStruc
%% Statistical analysis 3: effect of internal and external structures 
 Substrate = {'DM30'}; % substrate variable {'GL','CL','VA','DM10','DM20','DM30'}
 IntStruc = {'I1', 'I2', 'I3', 'I4'}; % Internal structure variable {'I1', 'I2', 'I3', 'I4'}
 ExtStruc = {'P', 'H', 'F'}; % External structure variable {'P', 'H', 'F'}
 
 [g,MaxF] = AnovaGrouping(Substrate,IntStruc,ExtStruc,UsedData);
 g1 = g(1,:);
 g2 = g(2,:);
 g3 = g(3,:);
 
 % Anova comparing, remove any grouping(g1,g2,g3) that containes only one
 % variable, adjust multcompare dimention acordingly
[~,Tbl3,Stats,~]=anovan(MaxF,{g2,g3},'model','full','varnames',{'Internal structure','Surface structure'},'Display','off'); 
[C3,M3,~,Gnames3] = multcompare(Stats,'dimension',[1,2],'Display','off'); 

clear  MaxF g g1 g2 g3 Stats  Substrate IntStruc ExtStruc

%% Statistical analysis 4:(adhesive samples on PVA) 
 Substrate = {'VA'}; % substrate variable {'GL','CL','VA','DM10','DM20','DM30'}
 IntStruc = {'I1', 'I2', 'I3', 'I4'}; % Internal structure variable {'I1', 'I2', 'I3', 'I4'}
 ExtStruc = {'P', 'H', 'F'}; % External structure variable {'P', 'H', 'F'}
 
 [g,MaxF] = AnovaGrouping(Substrate,IntStruc,ExtStruc,UsedData);
 g1 = g(1,:);
 g2 = g(2,:);
 g3 = g(3,:);
 
 % Anova comparing, remove any grouping(g1,g2,g3) that containes only one
 % variable, adjust multcompare dimention acordingly
[~,Tbl4,Stats,~]=anovan(MaxF,{g2,g3},'model','full','varnames',{'Internal structure','Surface structure'},'Display','off'); 
[C4,M4,~,Gnames4] = multcompare(Stats,'dimension',[2],'Display','off'); 

clear  MaxF g g1 g2 g3 Stats  Substrate IntStruc ExtStruc
%% Statistical analysis 5:(adhesive samples on cloth) 
 Substrate = {'CL'}; % substrate variable {'GL','CL','VA','DM10','DM20','DM30'}
 IntStruc = {'I1', 'I2', 'I3','I4'}; % Internal structure variable {'I1', 'I2', 'I3', 'I4'}
 ExtStruc = {'P', 'H', 'F'}; % External structure variable {'P', 'H', 'F'}
 
 [g,MaxF] = AnovaGrouping(Substrate,IntStruc,ExtStruc,UsedData);
 g1 = g(1,:);
 g2 = g(2,:);
 g3 = g(3,:);
 
 % Anova comparing remove any grouping(g1,g2,g3) that containes only one
 % variable, adjust multcompare dimention acordingly
[~,Tbl5,Stats,~]=anovan(MaxF,{g2,g3},'model','full','varnames',{'Internal structure','Surface structure'},'Display','off'); 
[C5,M5,~,Gnames5] = multcompare(Stats,'dimension',[1,2],'Display','off'); 

clear  MaxF g g1 g2 g3 Stats  Substrate IntStruc ExtStruc
%% Structures the data per condition used for creating boxplots.
k = 1;
VariableName = char(VariableList(1,1));
[VariableGroup(k).(VariableName)] = UsedData(1).maxforce;

for i = 2:length(AllData)
    % sorts the data per variable
    if isequal(VariableList(i),VariableList((i-1)))
        k = k+1;
        VariableGroup(k).(VariableName) = UsedData(i).maxforce;
    else
        VariableName = char(VariableList(i));
        k = 1;
        VariableGroup(k).(VariableName) = UsedData(i).maxforce;
    end
end

VariableList = fieldnames(VariableGroup);

clear i k VariableName 
%% calculate avarage maximum pull off force
 MeanF = zeros(length(VariableList),1);
 STD = zeros(length(VariableList),1);
 
for i = 1:length(VariableList)
   MeanF(i,:) = mean([VariableGroup.(VariableList{i})]);
   STD(i,:) = std([VariableGroup.(VariableList{i})]);
end

MeanForce = table(MeanF,STD,'RowNames',VariableList);

clear i MeanF STD
%% boxplot figures: effect internal structures on substrates with variing stiffness 
GroupVar1 = {'H'}; % The variable to be plotted in subplots, for example the substrate types  {'GL','CL','VA','DM10','DM20','DM30'}. 
GroupVar2 = {'GL','DM10', 'DM20', 'DM30'}; % select the variable to be grouped into subgroups, for example external structure type {'F','P','H'}; 
GroupVar3 = {'I1', 'I2', 'I3', 'I4'}; % select the variable to be plot in the subgroups, for example internal structure type {'I1', 'I2', 'I3', 'I4'};
Header = {};% Leave Empty if there are no subplots
GroupTitle = {''}; % array of subplot title(s)'Pull-off force unpatterned samples on substrates with decreasing stiffness'
YLabel = {'Friction force [N]'}; % y axis label
YLim = [0,9]; % set fixed limits to y axis (leave empty when fixed limits are undesired)
XLabel = {'I1','I2','I3','I4'}; % x axis tick labels 

SubGroupCheck = 1; % set to 1 if subgroup axis and labels are required (when GroupVar2 is longer than 1)
SubGroupLabel ={'Glass','PDMS 1:10', 'PDMS 1:20', 'PDMS 1:30'} ;%; % array of subgroup Label(s);

Plot(GroupVar1,GroupVar2,GroupVar3,Header,GroupTitle,YLabel,YLim,XLabel,SubGroupCheck,SubGroupLabel,VariableGroup,VariableList)
%% boxplot figures: effect external structures on substrates with variing stiffness
clear GroupVar1 GroupVar2 GroupVar3 Header GroupTitle XLabel YLabel SubGroupCheck SubGroupLabel YLim

GroupVar1 = {'I4'}; % The variable to be plotted in subplots, for example the substrate types {'GL','CL','VA','DM10','DM20','DM30'}. 
GroupVar2 = {'GL','DM10', 'DM20', 'DM30'}; % select the variable to be grouped into subgroups, for example external structure type {'F','P','H'}; 
GroupVar3 = {'F','P','H'}; % select the variable to be plot in the subgroups, for example internal structure type {'I1', 'I2', 'I3', 'I4'};
Header = {}; % Leave Empty if there are no subplots
GroupTitle = {''}; % array of subplot title(s)'Maximum pulloff force of I4 with differend surface patterns on substrates with decreasing stiffness'
YLabel = {'Friction force [N]'}; % y axis label
YLim = [0,9]; % set fixed limits to y axis (leave empty when fixed limits are undesired)
XLabel = {'Flat','Pillared','Hooked'}; % x axis tick labels 

SubGroupCheck = 1; % set to 1 if subgroup axis and labels are required (when GroupVar2 is longer than 1)
SubGroupLabel ={'Glass','PDMS 1:10', 'PDMS 1:20', 'PDMS 1:30'};%; % array of subgroup Label(s);

Plot(GroupVar1,GroupVar2,GroupVar3,Header,GroupTitle,YLabel,YLim,XLabel,SubGroupCheck,SubGroupLabel,VariableGroup,VariableList)

clear GroupVar1 GroupVar2 GroupVar3 Header GroupTitle XLabel YLabel SubGroupCheck SubGroupLabel YLim
%% boxplot figures: adhesive samples on Glass
GroupVar1 = {'GL'}; % The variable to be plotted in subplots, for example the substrate types  {'GL','CL','VA','DM10','DM20','DM30'}. 
GroupVar2 = {'F','P','H'}; % select the variable to be grouped into subgroups, for example external structure type {'F','P','H'}; 
GroupVar3 = {'I1', 'I2', 'I3', 'I4'}; % select the variable to be plot in the subgroups, for example internal structure type {'I1', 'I2', 'I3', 'I4'};
Header = {};
GroupTitle = {''}; % array of subplot title(s)'Pull off force of adhesives with internal and external structures on glass'
YLabel = {'Friction force [N]'}; % y axis label
YLim = [0,3.5]; % set fixed limits to y axis (leave empty when fixed limits are undesired)
XLabel = {'I1','I2','I3','I4'}; % x axis tick labels 

SubGroupCheck = 1; % set to 1 if subgroup axis and labels are required (when GroupVar2 is longer than 1)
SubGroupLabel = {'Flat','Pillared','Hooked'};%; % array of subgroup Label(s);

Plot(GroupVar1,GroupVar2,GroupVar3,Header,GroupTitle,YLabel,YLim,XLabel,SubGroupCheck,SubGroupLabel,VariableGroup,VariableList)

clear GroupVar1 GroupVar2 GroupVar3 Header GroupTitle XLabel YLabel SubGroupCheck SubGroupLabel YLim
%% boxplot figures: adhesive samples on PVA and PDMS 1:30
GroupVar1 = {'VA','DM30'}; % The variable to be plotted in subplots, for example the substrate types  {'GL','CL','VA','DM10','DM20','DM30'}. 
GroupVar2 = {'F','P','H'}; % select the variable to be grouped into subgroups, for example external structure type {'F','P','H'}; 
GroupVar3 = {'I1', 'I2', 'I3', 'I4'}; % select the variable to be plot in the subgroups, for example internal structure type {'I1', 'I2', 'I3', 'I4'};
Header = {}; %'Maximum friction force of adhesives with internal and external structures on PVA and PDMS 1:30'
GroupTitle = {'PVA substrate', 'PDMS 1:30 substrate'}; % array of subplot title(s)
YLabel = {'Friction force [N]'}; % y axis label
YLim = []; % set fixed limits to y axis (leave empty when fixed limits are undesired)
XLabel = {'I1','I2','I3','I4'}; % x axis tick labels 

SubGroupCheck = 1; % set to 1 if subgroup axis and labels are required (when GroupVar2 is longer than 1)
SubGroupLabel = {'Flat','Pillared','Hooked'};%; % array of subgroup Label(s);

Plot(GroupVar1,GroupVar2,GroupVar3,Header,GroupTitle,YLabel,YLim,XLabel,SubGroupCheck,SubGroupLabel,VariableGroup,VariableList)

clear GroupVar1 GroupVar2 GroupVar3 Header GroupTitle XLabel YLabel SubGroupCheck SubGroupLabel YLim
%% boxplot figures: adhesive samples on Cloth
GroupVar1 = {'CL'}; % The variable to be plotted in subplots, for example the substrate types  {'GL','CL','VA','DM10','DM20','DM30'}. 
GroupVar2 = {'F','P','H'}; % select the variable to be grouped into subgroups, for example external structure type {'F','P','H'}; 
GroupVar3 = {'I1', 'I2', 'I3', 'I4'}; % select the variable to be plot in the subgroups, for example internal structure type {'I1', 'I2', 'I3', 'I4'};
Header = {};
GroupTitle = {''}; % array of subplot title(s)'Pull off force of adhesives with internal and external structures on cloth'
YLabel = {'Friction force [N]'}; % y axis label
YLim = [0,3.5]; % set fixed limits to y axis (leave empty when fixed limits are undesired)
XLabel = {'I1','I2','I3','I4'}; % x axis tick labels 

SubGroupCheck = 1; % set to 1 if subgroup axis and labels are required (when GroupVar2 is longer than 1)
SubGroupLabel = {'Flat','Pillared','Hooked'};%; % array of subgroup Label(s);

Plot(GroupVar1,GroupVar2,GroupVar3,Header,GroupTitle,YLabel,YLim,XLabel,SubGroupCheck,SubGroupLabel,VariableGroup,VariableList)

clear GroupVar1 GroupVar2 GroupVar3 Header GroupTitle XLabel YLabel SubGroupCheck SubGroupLabel YLim

%% Functions used in the script

% moving avarage filterfunction
function outputsignal = maf(inputsignal,filterwindow)
% Zero-phase unweighted moving average filter function (using convolution equivalent of matlab's filtfilt() )

% Process inputs
x = inputsignal(:); % Enforce column vector
fw = filterwindow;

% Filter kernel
k = ones(1,fw)/fw; % Convolution kernel (sums to 1) [-]

% Forward-backward filtering using convolution
% Does not require signal processing toolbox
kk = conv(k,k); % Cascaded kernel (triangular, length 2*fw-1) [-]
nkk = length(kk); % Length of cascaded kernel
xc = [x(1)*ones((nkk-1)/2,1); x; x(end)*ones((nkk-1)/2,1)]; % Extrapolate boundaries by  (nkk-1)/2 samples to alleviate boundary problems
xcconv = conv(xc,kk); % Convolve signal with cascaded kernel
outputsignal = xcconv(nkk:end-nkk+1); % Select the part without the extrapolated boundaries
end

% sorting function used to sort the data that is used for ANOVA analysis 
function [g,MaxF] = AnovaGrouping(Substrate,IntStruc,ExtStruc,UsedData)
LA = length(Substrate)*length(IntStruc)*length(ExtStruc)*10;
Anova = struct('Name', cell(1,LA), 'Data', cell(1,LA)); 
MaxF = zeros(LA,1);
g{3,LA} = [];

m = 1;
 for i = 1:length(Substrate)
    for j = 1:length(IntStruc)
       for k = 1:length(ExtStruc)
        for l = 1:length(UsedData)
            if contains(UsedData(l).names,Substrate(i))*contains(UsedData(l).names,IntStruc(j))*contains(UsedData(l).names,ExtStruc(k)) == 1
                    
               Anova(m).Name = UsedData(l).names;
               Anova(m).Data = UsedData(l).maxforce;
               m = m + 1;
            end
        end
       end
    end
 end

  for i = 1:LA
   MaxF(i,:) =  Anova(i).Data;
   N = strsplit(Anova(i).Name,'_');
   g{1,i} = N{1,1}; % substrate group
   g{2,i} = N{1,2}; % internal structure group
   g{3,i} = N{1,3}; % external structure group
  end
end

% plot function used to create figures
function Plot(GroupVar1,GroupVar2,GroupVar3,Header,GroupTitle,YLabel,YLim,XLabel,SubGroupCheck,SubGroupLable,VariableGroup,VariableList)
    offset =0.05; % offset between the two x-axis 
    SDataF = struct();
    SNames = struct();
    %VariableList = fieldnames(VariableGroup);

    for k = 1:length(GroupVar2)% sort the selected data into the proper format suited for the boxplot
        for L = 1:length(GroupVar3) 
            for i = 1:length(VariableList)       
                if contains(VariableList(i),GroupVar2(k))*contains(VariableList(i),GroupVar3(L)) == 1
                    for j = 1:length(GroupVar1)
                        if contains(VariableList(i),GroupVar1(j))
                            if isfield(SDataF,(GroupVar1{j})) == 0
                                SDataF.(GroupVar1{j}) = [VariableGroup.(VariableList{i})]';
                                SNames.(GroupVar1{j}) = {VariableList{i}};
                            else
                                SDataF.(GroupVar1{j}) = [SDataF.(GroupVar1{j}),[VariableGroup.(VariableList{i})]'];
                                SNames.(GroupVar1{j}) = [SNames.(GroupVar1{j}),{VariableList{i}}];
                            end
                        end
                    end
                end
            end
        end
    end

    clear i j k L

    % Produce boxplots
    NSP = length(GroupVar1); % number of subplots
    NC = length(GroupVar2); % number of sub groups in a plot.
    NB = length(GroupVar3); % number of boxes per sub group
    NS = length(GroupVar2)*length(GroupVar3); % number samples per subplot;

    grad = 1/NB; % transparency gradient

    % creates XtickLabel array such that the subgroup labels are in the middle of the groups
    if SubGroupCheck == 1
        XSubLabel{1,(NC*2+1)} = [];
    
        for i = 1:NC
            XSubLabel{1,2*i} = SubGroupLable{i};
        end
    end

    figure()
    if isempty(Header) == 0
        Head = sgtitle(Header);
        Head.Margin = 100000;
    end

for i = 1:NSP
    subplot(length(GroupVar1),1,i) % Rows of subplots
    %subplot(2,2,i) % CAN BE USED to produce a custom layout.

    boxplot(SDataF.(GroupVar1{i}),SNames.(GroupVar1{i})) % change to SDataQ for area adjusted plots
    grid on

    if isempty(YLim) == 0
        ylim(YLim) % set fixed y axis values.
    end

    % fill the box with grayscale.
    a = get(get(gca,'children'),'children');   % Get the handles of all the objects
    t = {a.Tag}';   % List the names of all the objects
    k = 1;
    Box = gobjects(sum(contains(t,'Box')),1); 
        for j = 1:length(t) % select lines from 'a' with the 'Box' tag
           if contains(a(j).Tag,'Box')
                 Box(k,:) = a(j);
                 k = k+1;
           end 
        end
    clear k
    % changes the color of the boxes & add subgoup lines

    Ycoord = ylim;
    for k = 1:NB:NS
        if SubGroupCheck == 1 
            for l = 0:(NB-1)
                s = patch(Box((k+l)).XData(1:4),Box((k+l)).YData(1:4), 'k'); % Set the color of the first box to black
                alpha(s,grad*(l)) % adds transparancy 'grayscale'
            end
            for l = 0:(NC-2)
                Xcoord = [(NS/NC), (NS/NC)].*(l+1)+0.5;
                line([Xcoord(1) Xcoord(2)], [Ycoord(1),Ycoord(2)],'LineStyle','--','Color',[0 0 0]) % add lines to accentuate subgroups     
            end
        else
            s = patch(Box((k)).XData(1:4),Box((k)).YData(1:4), 'k'); % Set the color of the first box to black
            alpha(s,grad*(k-1)) % adds transparancy 'grayscale'      
        end
    end

    % add title, Y axis label and x- axis tick labels to the subplot.
    title((GroupTitle{i}));
    ylabel(YLabel)

    % add x-axis labels
    XLABEL = XLabel;
    for j=1:(NC-1)
        XLABEL = [XLABEL,XLabel];
    end
    set(gca,'xtick',1:NS,'xticklabel',XLABEL);

    % add subgroup labels if sub
    if SubGroupCheck == 1
        ax1 = gca; % current axes
        ax1_pos = ax1.Position; % position of first axes
        ax1_pos = [ax1_pos(1),ax1_pos(2)-offset,ax1_pos(3),ax1_pos(4)+offset]; % position subgroup axis
        ax2 = axes('Position',ax1_pos,'XAxisLocation','bottom','YAxisLocation','left','Color','none'); % creates subgroup x and y axis
        ax2.XLim = [0,2*NC]; % change the scale of the Subgroup x-axis
        ax2.XAxis.TickLabels = XSubLabel; % add correct labels to Subgroup x-axis
        ax2.YAxis.Visible = 'off'; % turns off subgroup y-axis
    end
end
end
