% Author: Frank Haring, Stud.nr. 4416937, Date 23-04-2019
clc, clear, close all

%% Load data from selected folder
DataList = dir(strcat(uigetdir,'\*.mat')); % list with .mat files in the selected folder.
AllData = struct(); 

for i = 1:length(DataList) % Extract data from the .mat files into 1 structure
    Data = load(fullfile(DataList(i).folder,DataList(i).name)); 
    Fields = fieldnames(load(fullfile(DataList(i).folder,DataList(i).name)));
    AllData.(Fields{1})(i)=Data.(Fields{1});
end
AllData = AllData.(Fields{1});
[AllData.names] = DataList.name; % add filenames to the structure

clear i
%% Select field to be used.
UsedFields = {'names','time_s','sensor_V','position_mm'}; % field names required for processing
UsedData = struct();

 for i = 1:length(DataList)
     for j =1:length(UsedFields)
        UsedData(i).(UsedFields{j}) = AllData(i).(UsedFields{j});
     end
 end
 
clear i j
 %% Cleaning Names (remove auto generated timestamp form the file names)
   for i = 1:length(UsedData)
    N = strsplit(UsedData(i).names,' '); % split name in parts;
    UsedData(i).names = strrep(string(strjoin(N(1:(4)))),' ','_'); %(removes auto generated time stamp from the name of the data)
    SampleList(i) = strrep(string(strjoin(N(1:(4)))),' ','_'); % creates sample namelist (ignores repetition code)
    VariableList(i) = strrep(string(strjoin(N(1:(4)))),' ','_'); % creates variable namelist (ignores repetitions and sample numbers)
   end 

clear i N
%% Remove noise using an moving avarage filter
fwfv = [41]; % Filter window voltage[-]
fwfp = [61]; % Filter window position-]

  for i = 1:length(UsedData)
        UsedData(i).sensor_V = maf(UsedData(i).sensor_V,fwfv(1))';
        UsedData(i).position_mm = maf(UsedData(i).position_mm,fwfp(1))';
  end    

clear i fwfv fwfp
%% Convert the signal from Volt to Newtons:
Offv = 201; % Number of samples used to remove offset.
C1 = 8.1654; % linear conversion constants determined for the 10 lb force sensor
C2 = 0.0587;
Vlim = 0.002; % lower limit to detect compression

  for i = 1:length(UsedData)
      % Dependend on the test different constants are to be used, determined by calibration.
     UsedData(i).force = C1.*UsedData(i).sensor_V - C2;
     
     % removes offset in voltage and position data 
       UsedData(i).force = (UsedData(i).force - mean(UsedData(i).force(1,1:Offv))); 
       UsedData(i).position_mm = (mean(UsedData(i).position_mm(1,1:Offv)) - UsedData(i).position_mm); 

       % extracts maximum force & compression data
      [UsedData(i).maxforce,UsedData(i).maxFindex] = max(UsedData(i).force(:)); % Extract maximum force. 
      
        FirstIndex = find(UsedData(i).sensor_V > Vlim,1); 
        LastIndex = UsedData(i).maxFindex;      
        
        UsedData(i).CompIndex = [FirstIndex LastIndex];
  end

clear i C1 C2 Offv FirstIndex LastIndex Vlim

 %% Calculate effective stiffness
fo1 = fitoptions('Method','NonlinearLeastSquares','Lower',[0,0],'Upper',[Inf,Inf],'StartPoint',[3 0.1]);
 myfittype1 = fittype(@(a,h,x) (1/(34.5))^(1/3).*real((x/a*(3/4)).^(2/3))+h,'options',fo1);

  for i = 1:length(UsedData)
COORD = UsedData(i).CompIndex;
Force = UsedData(i).force(1,(COORD(1):COORD(2)));
Compression = UsedData(i).position_mm(1,(COORD(1):COORD(2)));
f1 = fit((Force'), (Compression'), myfittype1);
 
UsedData(i).fit1 = f1;

UsedData(i).stiffness = [f1.a];

  end 
  
%% select a data number (i) to plot a fit.
i = 3;
COORD = UsedData(i).CompIndex;
Force = UsedData(i).force(1,(COORD(1):COORD(2)));
Compression = UsedData(i).position_mm(1,(COORD(1):COORD(2)));

figure(4)
plot(UsedData(i).fit1,(Force'),(Compression'))
grid on 
hold on
title('Hertz fit')
xlabel('force (N)')
ylabel('indentation (mm)')

  
 clear fo1 fo2 myfittype1 myfittype2 COORD Force Compression f1 f2 
%% Selects ands sorts the data to produce box plot. 
 structNames = {'D27_O1','D12_O2','Massive',};
 structNamesTemp = {'I1', 'I3','I4'};
 surfaceNames = {'Plain'};
 VelocityNames = {' '};
tcount = 0;
q = 1;
for i = 1:length(structNames)
    for j = 1:length(surfaceNames)
       for k = 1:length(VelocityNames)
           for L = 1:length(UsedData)
               
               X = strcat(structNamesTemp(i));
               
               if contains(UsedData(L).names,structNames(i))*contains(UsedData(L).names,surfaceNames(j)) == 1 %*contains(UsedData(L).names,VelocityNames(k)
                   
                   EmodFit1(q) = UsedData(L).stiffness(1,1);
                   NameList(q) = X;
                   
                   q = q+1;
               end
               
           end
       end
    end
end

figure(16)
boxplot(EmodFit1,NameList)
grid on
title ('effective normal stiffness Mpa (n/mm^2)')
ylabel ('E_{eff} (N/mm^2)')

%% Anova and multicompare:

[p,tbl,Stats] = anova1(EmodFit1,NameList);

figure()
[C,M,H,Gnames] = multcompare(Stats); % add or remove levels to plot different effects. 


%clear EmodFit1 Namelist tcount q
%% check calculate Emod based on volume fraction 
S = [85, 85]; 
Emod = (0.0981*(56+7.62336.*S))/(0.137505*(254-2.54.*S));

VolSolidPlain = 1360;
VolSolidTot = 1560;

VolPlain = [555.723, 642.463, 581.477, 642.464, 1360];
VolNames = {'D27 O1','D12 O1', 'D12 O2', 'D12 O3','Massive'};

VolFracPlain = VolPlain./VolSolidPlain;

EredPlain = Emod*(VolFracPlain);
clear S VolSolidPlain VolSolidTot VolPlain VolFracPlain 

%%
function outputsignal = maf(inputsignal,filterwindow)
% Zero-phase unweighted moving average filter function (using convolution equivalent of matlab's filtfilt() )

% Process inputs
x = inputsignal(:); % Enforce column vector
fw = filterwindow;

% Filter kernel
k = ones(1,fw)/fw; % Convolution kernel (sums to 1) [-]

% Forward-backward filtering using convolution
% Does not require signal processing toolbox
kk = conv(k,k); % Cascaded kernel (triangular, length 2*fw-1) [-]
nkk = length(kk); % Length of cascaded kernel
xc = [x(1)*ones((nkk-1)/2,1); x; x(end)*ones((nkk-1)/2,1)]; % Extrapolate boundaries by  (nkk-1)/2 samples to alleviate boundary problems
xcconv = conv(xc,kk); % Convolve signal with cascaded kernel
outputsignal = xcconv(nkk:end-nkk+1); % Select the part without the extrapolated boundaries
end