% Author: Frank Haring, Stud.nr. 4416937, Date 15-11-2019
clc, clear, close all

%% Load data from selected folder
DataList = dir(strcat(uigetdir,'\*.txt')); % list with .mat files in the selected folder.
AllData = struct(); 

for i = 1:length(DataList) % Extract data from the .mat files into 1 structure
    Data = readtable(fullfile(DataList(i).folder,DataList(i).name)); 
    Fields = Data.Properties.VariableNames;
    for j = 1:length(Fields)
    AllData(i).(Fields{j})=Data.(Fields{j});
    end
end

[AllData.names] = DataList.name; % add filenames to the structure
clear i j
%% Select field to be used.
UsedFields = {'names','Time_s_','Load_N_','Extension_mm_'}; % field names required for processing
UsedData = struct();

 for i = 1:length(DataList)
     for j =1:length(UsedFields)
        UsedData(i).(UsedFields{j}) = AllData(i).(UsedFields{j});
     end
 end
 
 clear i j
 %% Cleaning Names (remove file extension from the sample names)
   for i = 1:length(UsedData)
    N = strsplit(UsedData(i).names,'.'); % split name in parts;
    UsedData(i).names = N{1};%(removes file extention from the name of the data)
%    SampleList(i) = strrep(string(strjoin(N(1:(4)))),' ','_'); % creates sample namelist (ignores repetition code)
%    VariableList(i) = strrep(string(strjoin(N(1:(4)))),' ','_'); % creates variable namelist (ignores repetitions and sample numbers)
   end 
   
clear i N
%% Cleaning Load and Extension (removes NaN and corrects Extention)
for i = 1:length(UsedData)
    UsedData(i).Load_N_(isnan(UsedData(i).Load_N_)) = 0; % removes NaN from data
    UsedData(i).Extension_mm_(isnan(UsedData(i).Extension_mm_)) = 0; % removes NaN from data
    
    k = find(abs(UsedData(i).Extension_mm_),1); % starting index for startpoint correction
    Offset = UsedData(i).Extension_mm_(k);
    UsedData(i).Stiffness = zeros(1000,1);
    
    
    for j = k:length(UsedData(i).Extension_mm_)
    UsedData(i).Extension_mm_(j) = UsedData(i).Extension_mm_(j) - Offset; % corrects extention for varing starting positions of the tests.

    end
    
    [UsedData(i).MaxForce,UsedData(i).MaxIndex] = max(UsedData(i).Load_N_(:)); % Extract maximum force.
     UsedData(i).MaxExtension = UsedData(i).Extension_mm_(UsedData(i).MaxIndex);
    UsedData(i).FirstIndex = find(UsedData(i).Extension_mm_>0.4,1);
end

clear i
%% Determine the stiffness increments
for i = 1:length(UsedData)
    
    %k = find(abs(UsedData(i).Extension_mm_),1); % starting index for startpoint correction
    UsedData(i).Stiffness = zeros(UsedData(i).MaxIndex,1);
    
    for j = UsedData(i).FirstIndex:UsedData(i).MaxIndex % from the first nonzero element until peak force
        
        dF = UsedData(i).Load_N_(j-1)-UsedData(i).Load_N_(j); % Force Increment
    	dX = UsedData(i).Extension_mm_(j-1)-UsedData(i).Extension_mm_(j); % Extention increment
        UsedData(i).Stiffness(j) = dF/dX; % Incremental stiffness
    end
end

%% stiffness
B = 5; % number of bins
for i = 1:length(UsedData)
[N,edges,bin] = histcounts(UsedData(i).Stiffness,B);

for n = 1:B
    UsedData(i).BinMean(:,n) = mean(UsedData(i).Stiffness(bin==n,1))'; %Avarage stiffness within each bin
end
[UsedData(i).MaxStiff,UsedData(i).MaxStiffIndex] = max(UsedData(i).Stiffness);
end
%% Selects ands sorts the data to produce box plot. 
 structNames = {'I1','I2','I3',};
 structNamesTemp = {'I1', 'I2','I3'};
  surfaceNames = {'P', 'H', 'F'}; %{'P', 'H', 'F'}
tcount = 0;
q = 1;
for i = 1:length(structNames)
    for j = 1:length(surfaceNames)
       for L = 1:length(UsedData)
               
               X = strcat(structNamesTemp(i)); % USE strcat(structNamesTemp(i),'_',surfaceNames(j)); to fiter for surface structures !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
               
               if contains(UsedData(L).names,structNames(i))*contains(UsedData(L).names,surfaceNames(j)) == 1 %
                   
                   Stiffnes(q) = UsedData(L).MaxStiff;
                   NameList(q) = X;
                   
                   q = q+1;
               end
               
           end
       end
    end

figure(16)
boxplot(Stiffnes,NameList)
grid on
title ('effective shear stiffness Mpa (n/mm^2)')
ylabel ('Stiffness (N/mm^2)')

%% Anova and multicompare:

[p,tbl,Stats] = anova1(Stiffnes,NameList);

figure()
[C,M,H,Gnames] = multcompare(Stats); % add or remove levels to plot different effects. 

%% images
i = 2; % sample number to be plotted 

figure(1)
plot(UsedData(i).Time_s_,UsedData(i).Load_N_,'.')
xlabel('time [s]')
ylabel('Force [N]')
grid on
title('Force-time plot')
figure(2)
plot(UsedData(i).Time_s_,UsedData(i).Extension_mm_,'.')
grid on
xlabel('time [s]')
ylabel('Extention [mm]')
title('Extention-time plot')

figure(3)
plot(UsedData(i).Extension_mm_,UsedData(i).Load_N_,'.')
grid on
xlabel('Extention [mm]')
ylabel('Force [N]')
title('Force-Extention plot')

figure(4)
plot(UsedData(i).Extension_mm_(1:UsedData(i).MaxIndex),UsedData(i).Stiffness,'.')
grid on
hold on
plot(UsedData(i).Extension_mm_(UsedData(i).MaxStiffIndex),UsedData(i).MaxStiff,'o','LineWidth',1.5,'MarkerSize',10);
text((UsedData(i).Extension_mm_(UsedData(i).MaxStiffIndex)-0.2),UsedData(i).MaxStiff+2,'Maximum stiffness')
xlabel('Extention [mm]')
ylabel('Stiffness [N/mm]')
title('Incremental shear stiffness')
